@@ -161,3 +161,15 @@ class LnItemRsnsNotUpdatedException(ProcessException):
     """LnItemRsns Not Updated Exception."""
 
     pass
+
+
+class StatusNotUpdatedException(ProcessException):
+    """Status Not Updated Exception."""
+
+    pass
+
+
+class MaxScrollTriesException(Exception):
+    """Max Scroll Tries Exception."""
+
+    pass
