@@ -4,7 +4,7 @@ import contextlib
 from logging import Logger
 import time
 from abc import ABCMeta
-from typing import Optional
+from typing import Optional, Any
 import pyautogui
 import _ctypes
 from retry import retry
@@ -19,6 +19,7 @@ if IS_WINDOWS_OS:
     from pywinauto.findwindows import ElementNotFoundError
     from pywinauto.controls.hwndwrapper import InvalidWindowHandle
     from pywinauto.controls.uia_controls import ListItemWrapper
+    from pywinauto import mouse
 
 
 from t_nextgen.utils.logger import logger as lib_logger
@@ -80,7 +81,7 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
         modal_title: str = "NextGen",
         buttons_to_try: list[str] = ["OK"],
         timeout: int = 5,
-        retry_interval: int = 2,
+        retry_interval: int = 0.001,
     ) -> tuple[bool, str]:
         """Close a modal in the app by clicking on the given button and returning the modal text.
 
@@ -92,9 +93,9 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
             retry_interval (int): The control is checked for existence this number of seconds.
 
         """
-        modal = self.dialog.child_window(title=modal_title, control_type="Window")
+        modal = self.dialog.child_window(title=modal_title, control_type="Window", top_level_only=True)
         if modal.exists(timeout, retry_interval):
-            modal_text_control = modal.child_window(control_type="Text")
+            modal_text_control = modal.child_window(control_type="Text", top_level_only=True)
             modal_text = modal_text_control.window_text()
 
             # Find and click the button
@@ -115,7 +116,7 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
     def connect_to_logged_in_screen(self) -> None:
         """Connect to the Next Gen App after it has been logged into."""
         try:
-            self.connect_to_app(title="MainForm", timeout=20)
+            return self.connect_to_app(title="MainForm", timeout=20)
         except Exception as e:
             self.click_on_next_gen_window()
             raise e
@@ -136,8 +137,8 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
     @retry_if_pywin_error()
     def open_file_menu(self) -> None:
         """This function opens the file menu."""
-        file_menu = self.dialog.child_window(title="File", control_type="MenuItem", found_index=0)
-        self.mouse_click_element(file_menu)
+        file_menu = self.dialog.child_window(title="File", control_type="MenuItem", top_level_only=True, found_index=0)
+        file_menu.click_input()
         self.dialog.child_window(title="FileDropDown").wait("visible", 30)
 
     @retry((TimeoutError, OSError), 3, 1)
@@ -169,7 +170,7 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
                 self.deal_with_unhandled_exception_popup(f"NextGen - {practice}")
                 self.click_no_button_in_next_gen_alert()
                 closed_windows_count += 1
-            except Exception as e:
+            except (_ctypes.COMError, RuntimeError, AttributeError) as e:
                 self.logger.error(f"Error closing window: {str(e)}")
 
         self.logger.debug(f"Closed {closed_windows_count} child windows.")
@@ -193,7 +194,7 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
             return True
         try:
             modal = self.dialog.child_window(title="NextGen", control_type="Window")
-            if modal.exists(timeout=modal_exists_timeout, retry_interval=1):
+            if modal.exists(timeout=modal_exists_timeout, retry_interval=0.001):
                 wrapper_object = modal.wrapper_object()
                 buttons_list = wrapper_object.descendants(control_type="Button")
 
@@ -228,12 +229,73 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
         except (TimeoutError, ElementNotFoundError, IndexError):
             return False
 
+    def click_ok_popup_getting_message(
+        self, call_count: int = 0, max_calls: int = 3, modal_exists_timeout: int = 3, sleep: float = 5
+    ) -> str:
+        """Clicks the "OK" button on a modal if it is the only option and checks for additional modals.
+
+        This function searches for a modal dialog with the title "NextGen". If found, it looks for "OK",
+        "CLOSE", and "CANCEL" buttons. If "OK" is the only available button, it clicks it. The function
+        will recursively check for additional modals up to a maximum of `max_calls`. And return the message
+        in the modal.
+
+        Args:
+            call_count (int, optional): _description_. Defaults to 0.
+            max_calls (int, optional): _description_. Defaults to 3.
+            modal_exists_timeout (int, optional): _description_. Defaults to 3.
+            sleep (float, optional): _description_. Defaults to 5.
+
+        Returns:
+            str: The text message from the modal dialog, or an empty string if modal was not found.
+        """
+        if call_count >= max_calls:
+            return ""
+        try:
+            modal = self.dialog.child_window(title="NextGen", control_type="Window")
+            if modal.exists(timeout=modal_exists_timeout, retry_interval=0.001):
+                wrapper_object = modal.wrapper_object()
+                buttons_list = wrapper_object.descendants(control_type="Button")
+
+                ok_button = None
+                close_button = None
+                more_buttons = False
+                for button in buttons_list:
+                    button_text = button.texts()[0].upper()
+                    if button_text == "OK":
+                        ok_button = button
+                    elif button_text in ["CLOSE", "CANCEL"]:
+                        close_button = button
+                    else:
+                        more_buttons = True
+                if more_buttons:
+                    return ""
+                if ok_button and close_button:
+                    self.logger.debug("Ok button found. Clicking on it.")
+                    texts = wrapper_object.descendants(control_type="Text")
+                    modal_text = None
+                    if texts:
+                        modal_text = texts[0].texts()
+                        self.logger.debug(f"Text in the modal: {modal_text}")
+                    with contextlib.suppress(_ctypes.COMError):
+                        ok_button.click()
+                    time.sleep(sleep)
+                    self.click_ok_popup_getting_message(
+                        call_count=call_count + 1, max_calls=max_calls
+                    )  # Recursive call with incremented call_count
+                    return modal_text[0] if modal_text else ""
+            else:
+                return ""
+        except _ctypes.COMError:
+            return ""
+        except (TimeoutError, ElementNotFoundError, IndexError):
+            return ""
+
     def click_no_button_in_next_gen_alert(self) -> None:
         """Clicks the "No" button on the NextGen alert window if it exists."""
         no_button = self.dialog.child_window(title="NextGen", control_type="Window").child_window(
             auto_id="7", control_type="Button"
         )
-        if no_button.exists(timeout=3, retry_interval=1):
+        if no_button.exists(timeout=1, retry_interval=0.001):
             with contextlib.suppress(_ctypes.COMError):
                 no_button.click()
             no_button.wait_not("visible", timeout=5)
@@ -293,6 +355,31 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
         for _ in range(n + confidence_value):
             pyautogui.press("down")
 
+    def click_center_and_scroll(
+        self, element: WindowSpecification, index: int, page_size: int = 50, scroll_step: int = 3
+    ) -> None:
+        """Clicks at the center of the given element and performs a downward scroll.
+
+        Args:
+            element: The element on which the click and scroll action is performed.
+            index (int): The index to determine the scroll amount.
+            page_size (int, optional): The size of the page to calculate scrolling. Defaults to 50
+            scroll_step (int, optional): The step size for each scroll action. Defaults to 3.
+
+        Raises:
+            Exception: If any error occurs during the click or scroll operation,
+        """
+        rect = element.rectangle()
+        center_x = rect.left + (rect.width() // 2)
+        center_y = rect.top + (rect.height() // 2)
+        mouse.click("left", (center_x, center_y))
+        pyautogui.press("home")
+        scroll_count = max(0, (index - page_size + 1 + scroll_step - 1) // scroll_step)
+        self.logger.debug(f"Index to scroll: {index}")
+        self.logger.debug(f"Scroll count {scroll_count}")
+        mouse.scroll(coords=(center_x, center_y), wheel_dist=-scroll_count)
+        time.sleep(0.5)
+
     def click_button_by_element_name(self, element_name: str) -> None:
         """This method clicks on a button by element name.
 
@@ -401,15 +488,24 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
         with contextlib.suppress(_ctypes.COMError):
             combobox.click_input()
 
-    def deal_with_unhandled_exception_popup(self, windows_title: str = "Microsoft .NET Framework"):
-        """This method deals with the unhandled exception popup."""
+    def deal_with_unhandled_exception_popup(self, windows_title: str = "Microsoft .NET Framework") -> bool:
+        """This method deals with the unhandled exception popup.
+
+        Args:
+            windows_title (str): The title of the window to check for.
+
+        Returns:
+            bool: True if the popup was found and handled, False otherwise.
+        """
         with contextlib.suppress(ElementNotFoundError, InvalidWindowHandle):
             popup_app = Application(backend="win32").connect(title=windows_title)
             top_window = popup_app.top_window()
-            if top_window.exists(timeout=2, retry_interval=1):
+            if top_window.exists(timeout=1, retry_interval=0.001):
                 top_window.child_window(title="&Continue").click()
+                return True
+            return False
 
-    def get_element_coordinates(self, element) -> tuple:
+    def get_element_coordinates(self, element: WindowSpecification) -> tuple:
         """This method gets the element coordinates.
 
         Args:
@@ -420,7 +516,7 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
         """
         return element.rectangle().mid_point()
 
-    def safe_wait(self, kwargs):
+    def safe_wait(self, kwargs: Any) -> bool | TimeoutError:
         """Waits for an element specified by `kwargs` to become visible in the `next_gen` dialog window.
 
         This function will attempt to wait for an element to become visible up to a maximum of 3 tries.
@@ -487,7 +583,7 @@ class NGAppManager(DesktopApp, metaclass=ScopedSingletonMeta):
                 field.set_text(text)
                 return
             except _ctypes.COMError:
-                time.sleep(1)
+                time.sleep(0.5)
                 if field.get_value() == text:
                     return
             retries += 1
