@@ -78,56 +78,56 @@ class NextGenPMCore:
         self.desktop_app = NGAppManager(app_path, self.logger)
 
     @property
-    def batch_posting_window(self):
+    def batch_posting_window(self) -> BatchPostingWindow:
         """This property is used to access BatchPostingWindow class."""
         if self._batch_posting_window is None:
             self._batch_posting_window = BatchPostingWindow(self.app_path, self.logger)
         return self._batch_posting_window
 
     @property
-    def batch_maintenance_window(self):
+    def batch_maintenance_window(self) -> BatchMaintenanceWindow:
         """This property is used to access BatchMaintenanceWindow class."""
         if self._batch_maintenance_window is None:
             self._batch_maintenance_window = BatchMaintenanceWindow(self.app_path, self.logger)
         return self._batch_maintenance_window
 
     @property
-    def balance_control_window(self):
+    def balance_control_window(self) -> BalanceControlWindow:
         """This property is used to access BatchMaintenanceWindow class."""
         if self._balance_control_window is None:
             self._balance_control_window = BalanceControlWindow(self.app_path, self.logger)
         return self._balance_control_window
 
     @property
-    def cob_window(self):
+    def cob_window(self) -> COBWindow:
         """This property is used to access COBWindow class."""
         if self._cob_window is None:
             self._cob_window = COBWindow(self.app_path, self.logger)
         return self._cob_window
 
     @property
-    def encounter_insurance_window(self):
+    def encounter_insurance_window(self) -> EncounterInsuranceWindow:
         """This property is used to access EncounterInsuranteWindow class."""
         if self._encounter_insurance_window is None:
             self._encounter_insurance_window = EncounterInsuranceWindow(self.app_path, self.logger)
         return self._encounter_insurance_window
 
     @property
-    def import_file_window(self):
+    def import_file_window(self) -> ImportFileWindow:
         """This property is used to access ImportFileWindow class."""
         if self._import_file_window is None:
             self._import_file_window = ImportFileWindow(self.app_path, self.logger)
         return self._import_file_window
 
     @property
-    def payment_entry_window(self):
+    def payment_entry_window(self) -> PaymentEntryWindow:
         """This property is used to access PaymentEntryWindow class."""
         if self._payment_entry_window is None:
             self._payment_entry_window = PaymentEntryWindow(self.app_path, self.logger, self.textract)
         return self._payment_entry_window
 
     @property
-    def transaction_ledger_window(self):
+    def transaction_ledger_window(self) -> TransactionLedgerWindow:
         """This property is used to access TransactionLedgerWiundow class."""
         if self._transaction_legder_window is None:
             self._transaction_legder_window = TransactionLedgerWindow(self.app_path, self.logger)
@@ -138,11 +138,16 @@ class NextGenPMCore:
     def login(
         self,
         practice: Optional[str] = None,
-        database: Optional[str] = None,
+        database: Optional[str | list[str]] = None,
         app_folder: str = Config.DIRECTORIES.BASE_DIR,
     ) -> None:
         """Login to the app."""
-        database = database if database else self.database
+        if not database:
+            databases = [self.database]
+        elif isinstance(database, str):
+            databases = [database]
+        else:
+            databases = database
         practice = practice if practice else self.practice
         if process := self.desktop_app.get_app_session_if_running(self.app_path):
             self.logger.debug(f"Process {process} is already running.")
@@ -151,7 +156,7 @@ class NextGenPMCore:
         try:
             self.desktop_app.start_app(app_path=self.app_path, app_folder=app_folder)
             self.desktop_app.wait_until_element_visible(auto_id="txtUserName", control_type="Edit")
-            self._check_database_name(db_name_flag=database)
+            self._check_database_names(db_names=databases)
             if not self.desktop_app.dialog.child_window(title="Windows Integrated", control_type="ComboBox").exists():
                 self.desktop_app.set_input_text(auto_id="txtUserName", text=self.username)
                 self.desktop_app.set_input_text(auto_id="txtPassword", text=self.password)
@@ -188,6 +193,23 @@ class NextGenPMCore:
         if db_name_flag.lower() not in database_name.lower():
             raise WrongDatabaseSelectedError(f"'{database_name}' selected, wrong database selected.")
 
+    def _check_database_names(self, db_names: list[str]) -> None:
+        """Validates if the name received is part of the database selected.
+
+        Args:
+            db_names (list[str]): The database names to be validated.
+
+        Returns:
+            Raises WrongDatabaseSelected: if the database doesn't match.
+        """
+        for db_name in db_names:
+            try:
+                self._check_database_name(db_name)
+                return
+            except WrongDatabaseSelectedError:
+                continue
+        raise WrongDatabaseSelectedError(f"None of the databases '{db_names}' selected.")
+
     def handle_multiple_sessions_popup(self) -> None:
         """Close multiple sessions pop-up that comes immediately after login.
 
@@ -195,14 +217,15 @@ class NextGenPMCore:
             MultipleSessionsError: if the modal exists
         """
         if self.desktop_app.dialog.child_window(title="Multiple station logon error", control_type="Window").exists(
-            timeout=3, retry_interval=1
+            timeout=2, retry_interval=0.001
         ):
             self.logger.warning("Multiple sessions detected. Closing the popup.")
             if next_gen_process := self.desktop_app.get_app_session_if_running(self.app_path):
                 self.close_session(next_gen_process)
             raise MultipleSessionsError("Process failed with multiple sessions issue")
 
-    def select_practice_from_app(self, practice: str):
+    @capture_screenshot_if_pywin_error()
+    def select_practice_from_app(self, practice: str) -> None:
         """Select practice from the Health Practice menu.
 
         Args:
@@ -211,7 +234,7 @@ class NextGenPMCore:
         self.click_practice_from_bar()
         self.input_check_practice(practice)
 
-    def check_if_practice_was_set_correctly(self, practice: str):
+    def check_if_practice_was_set_correctly(self, practice: str) -> None:
         """Check if the practice was set correctly.
 
         Args:
@@ -225,7 +248,7 @@ class NextGenPMCore:
             )
 
     @retry(tries=3, delay=3)
-    def click_practice_from_bar(self):
+    def click_practice_from_bar(self) -> None:
         """Clicks the Health Practice button from the menu bar."""
         practice_icon = self.desktop_app.dialog.child_window(title="cmdToolPractice", control_type="Button")
         with contextlib.suppress(_ctypes.COMError):
@@ -235,7 +258,7 @@ class NextGenPMCore:
         self.desktop_app.dialog.child_window(auto_id="cboPractice", control_type="Pane").wait("visible", 5)
 
     @retry(tries=3, delay=3)
-    def input_check_practice(self, practice: str):
+    def input_check_practice(self, practice: str) -> None:
         """Input check practice information and click OK.
 
         Args:
@@ -263,7 +286,7 @@ class NextGenPMCore:
         self.logger.info("Running close_session method.")
         try:
             self.desktop_app.connect_to_main_form()
-            self.desktop_app.close_modal(buttons_to_try=["NO", "OK"])
+            self.desktop_app.close_modal(buttons_to_try=["NO", "OK"], timeout=1)
             self.desktop_app.clear_out_windows_function(practice=practice)
             self.desktop_app.open_file_menu()
             logout_key = "l"
@@ -275,8 +298,8 @@ class NextGenPMCore:
 
         # This kind of .exe needs to kill the app after logout
         try:
-            self.desktop_app.close_modal(buttons_to_try=["YES"])
-            time.sleep(20)
+            self.desktop_app.close_modal(buttons_to_try=["YES"], timeout=2)
+            time.sleep(10)
             self.desktop_app.wait_to_disappear(auto_id="cmdToolApptlist", control_type="Button", max_attempts=12)
             self.logger.info("Session closed successfully.")
         except (AssertionError, RuntimeError):
@@ -284,5 +307,5 @@ class NextGenPMCore:
                 raise AppCrashException("Session not closed successfully")
             self.logger.warning("Session not closed successfully, ignoring the error.")
         self.logger.info("Killing the process for EnterprisePM.exe")
-        time.sleep(5)
+        time.sleep(3)
         self.desktop_app.kill_app(process=process, app_path=self.app_path)
