@@ -6,7 +6,6 @@ from decimal import Decimal
 
 import _ctypes
 from typing import Optional
-import pyautogui
 from retry import retry
 from t_desktop.config import IS_WINDOWS_OS
 from t_desktop.utils.capture_screenshot import capture_screenshot
@@ -16,6 +15,7 @@ if IS_WINDOWS_OS:
     from pywinauto.controls.uia_controls import ListItemWrapper
     from pywinauto.application import WindowSpecification
     from pywinauto.findwindows import ElementNotFoundError
+    from pywinauto.timings import TimeoutError
 from t_desktop.decorators import retry_if_pywin_error
 from t_nextgen.nextgen_window import NextGenWindow
 from t_nextgen.constants import CONST
@@ -32,13 +32,13 @@ class BatchPostingWindow(NextGenWindow):
     """Batch Posting Class with methods to interact with batch posting window."""
 
     @property
-    def window(self):
+    def window(self) -> WindowSpecification:
         """Return the AMBatches window element."""
         self.desktop_app.dialog.child_window(auto_id="NGEPMBatchLookup", control_type="Window").wait("visible", 5)
         return self.desktop_app.dialog.child_window(auto_id="NGEPMBatchLookup", control_type="Window")
 
     @property
-    def batch_posting_window(self):
+    def batch_posting_window(self) -> WindowSpecification:
         """Return the BatchPosting window element."""
         return self.desktop_app.dialog.child_window(title="BatchPosting", control_type="Window")
 
@@ -48,15 +48,13 @@ class BatchPostingWindow(NextGenWindow):
         try:
             self.logger.debug("Clicking on the batch icon from the bar.")
             batch_icon = self.desktop_app.dialog.child_window(auto_id="cmdToolPosting")
-            self.desktop_app.mouse_click_element(batch_icon)
+            batch_icon.click()
             self.desktop_app.safe_wait({"auto_id": "Data Area", "control_type": "Custom", "found_index": 0})
-            self.maximize_batch_window()
         except _ctypes.COMError:
             pass
-        except TimeoutError as e:
-            self.desktop_app.deal_with_unhandled_exception_popup(f"NextGen - {practice_name}")
-            raise e
-        except ElementNotFoundError:
+        except (ElementNotFoundError, TimeoutError, RuntimeError):
+            if self.desktop_app.deal_with_unhandled_exception_popup(f"NextGen - {practice_name}"):
+                raise RuntimeError("Unhandled Exception popup appeared")
             raise BatchFromBarNotFound("Element with auto_id: cmdToolPosting, not found")
 
     def post_batch(self, row: ListItemWrapper, post_the_batch: bool = True) -> bool:
@@ -91,7 +89,6 @@ class BatchPostingWindow(NextGenWindow):
             bool: True if the window menu is found, False otherwise
         """
         windows_menu = self.desktop_app.dialog.child_window(title="Window", control_type="MenuItem")
-        windows_menu.set_focus()
         windows_menu.select()
         try:
             windows_menu.child_window(title="1 Batch Posting", control_type="MenuItem").select()
@@ -104,7 +101,7 @@ class BatchPostingWindow(NextGenWindow):
 
     def _deal_with_no_batch_found_popup(self) -> None | NoMatchingBatchDescriptionException:
         modal = self.desktop_app.dialog.child_window(title="NextGen", control_type="Window")
-        if modal.exists(timeout=3, retry_interval=1):
+        if modal.exists(timeout=2, retry_interval=0.001):
             self.logger.debug("No batch found. Closing popup")
             send_keys("{ENTER}")
             send_keys("%C")
@@ -114,7 +111,6 @@ class BatchPostingWindow(NextGenWindow):
     def _set_amount_field_in_batch_lookup(self, search_criteria_window: WindowSpecification, amount: Decimal) -> None:
         self.logger.debug(f"Setting amount field in batch lookup to {amount}")
         amount_field = search_criteria_window.child_window(auto_id="txtAmount", control_type="Edit")
-        amount_field.set_focus()
         self.desktop_app.set_text(amount_field, amount)
 
     def find_batch(
@@ -130,6 +126,7 @@ class BatchPostingWindow(NextGenWindow):
         batch_row = self.run_batch_lookup(description, amount, fill_amount)
         return batch_row.row_element
 
+    @retry(TimeoutError, tries=3, delay=1)
     def run_batch_lookup(self, trn: str, amount: Optional[Decimal], fill_amount: bool = False) -> BatchPostingRow:
         """Performs a batch lookup based on the provided criteria.
 
@@ -148,17 +145,16 @@ class BatchPostingWindow(NextGenWindow):
         """
         self.logger.info(f"Running batch lookup: trn={trn}, amount={amount}")
         self.click_menu_icon("a")
-        search_criteria_window = self.window.child_window(auto_id="grpSearchCriteria")
+        search_criteria_window = self.window.child_window(auto_id="grpSearchCriteria", control_type="Group")
         if fill_amount:
             self._set_amount_field_in_batch_lookup(search_criteria_window, amount)
         description_field = search_criteria_window.child_window(auto_id="txtDesc", control_type="Edit")
-        description_field.set_focus()
         self.desktop_app.set_text(description_field, trn)
         send_keys("%F")
 
         self._deal_with_no_batch_found_popup()
 
-        tree_view = self.window.child_window(auto_id="ColScrollRegion: 0, RowScrollRegion: 0")
+        tree_view = self.window.child_window(auto_id="ColScrollRegion: 0, RowScrollRegion: 0", control_type="Tree")
 
         filtered_rows = self.get_batch_posting_rows(tree_view)
         if filtered_rows is None:
@@ -171,7 +167,7 @@ class BatchPostingWindow(NextGenWindow):
         if rows_count == 1:
             row: BatchPostingRow = rows[0]
             if not row.visible:
-                self._click_down_n_times(row.index)
+                self.desktop_app.click_center_and_scroll(self.batch_posting_window, row.index, 10)
                 row.visible = True
             with contextlib.suppress(_ctypes.COMError):
                 row.edit_wrapper.click_input()
@@ -196,8 +192,7 @@ class BatchPostingWindow(NextGenWindow):
         self.logger.debug("Clicking the menu icon button in the 'Batch Posting' window")
         self.batch_window = self._get_batch_window()
         cmd_drill_button = self.batch_window.child_window(title="cmdDrill", control_type="Button")
-        with contextlib.suppress(_ctypes.COMError):
-            cmd_drill_button.click_input()
+        cmd_drill_button.click_input()
         time.sleep(1)
 
         send_keys(key)
@@ -210,8 +205,7 @@ class BatchPostingWindow(NextGenWindow):
             WindowSpecification: The 'Batch Posting' window object.
         """
         self.logger.debug("Getting the 'Batch Posting' window")
-        self.desktop_app.wait_until_element_visible(control_type="Window", title="BatchPosting", timeout=15)
-        self.desktop_app.wait_until_element_visible(control_type="Pane", title="lstListing", timeout=15)
+        self.desktop_app.wait_until_element_visible(control_type="Pane", title="lstListing", timeout=1)
         batch_window = self.desktop_app.dialog.child_window(title="BatchPosting", control_type="Window")
         return batch_window
 
@@ -289,18 +283,6 @@ class BatchPostingWindow(NextGenWindow):
             return amount_str
         return None
 
-    def _click_down_n_times(self, n: int, confidence_value: int = 3) -> None:
-        """This method clicks the down key n times.
-
-        Args:
-            n (int): The number of times to click the down key.
-            confidence_value (int, optional): confidence value. Defaults to 3.
-        """
-        self.logger.debug(f"Clicking down {n + confidence_value} times")
-        pyautogui.press("home")
-        for _ in range(n + confidence_value):
-            pyautogui.press("down")
-
     def check_boxes_in_batch_posting(self, current_phase: str) -> None:
         """Toggle the state of three checkboxes in a batch posting dialog.
 
@@ -327,8 +309,25 @@ class BatchPostingWindow(NextGenWindow):
             else:
                 self.desktop_app.toggle_checkbox(checkbox)
 
+    def unselect_checkboxes(self) -> None:
+        """Untoggle three checkboxes in a batch posting dialog.
+
+        UnToggles checkboxes with auto_ids "_chkSecure_0", "_chkSecure_1", and "_chkSecure_2"
+        in the current dialog.
+
+        """
+        checkbox_ids = ["_chkSecure_0", "_chkSecure_1", "_chkSecure_2"]
+        checkboxes = []
+        self.desktop_app.dialog.child_window(auto_id=checkbox_ids[0], control_type="CheckBox").wait("visible", 5)
+
+        for checkbox_id in checkbox_ids:
+            checkboxes.append(self.desktop_app.dialog.child_window(auto_id=checkbox_id, control_type="CheckBox"))
+
+        for checkbox in checkboxes:
+            self.desktop_app.untoggle_checkbox(checkbox)
+
     @retry(tries=3, delay=3)
-    def set_focus_in_batch_window(self):
+    def set_focus_in_batch_window(self) -> None:
         """This method sets the focus in the batch window."""
         self.logger.debug("Setting focus in the batch window")
         window = self.desktop_app.dialog.child_window(title="BatchPosting", control_type="Window")
@@ -369,8 +368,8 @@ class BatchPostingWindow(NextGenWindow):
             if not self.batch_posting_window.was_maximized():
                 self.logger.debug("Maximizing the batch window")
                 self.batch_posting_window.maximize()
-            time.sleep(2)
-        except Exception as e:
+                time.sleep(1)
+        except (_ctypes.COMError, RuntimeError, AttributeError) as e:
             self.logger.warning(f"Could not maximize the batch window: {str(e)}")
 
     def get_batch_row(self, trn: str, amount: str) -> None | BatchPostingRow:
@@ -401,7 +400,7 @@ class BatchPostingWindow(NextGenWindow):
         if rows_count == 1:
             row: BatchPostingRow = rows[0]
             if not row.visible:
-                self._click_down_n_times(row.index)
+                self.desktop_app.click_center_and_scroll(self.batch_posting_window, row.index, 10)
                 row.visible = True
             with contextlib.suppress(_ctypes.COMError):
                 row.edit_wrapper.click_input()
