@@ -5,6 +5,8 @@ from t_desktop.config import IS_WINDOWS_OS
 if IS_WINDOWS_OS:
     from pywinauto.application import WindowSpecification
     from pywinauto.controls.uia_controls import ListItemWrapper
+    from pywinauto.timings import TimeoutError
+
 from typing import Any
 import _ctypes
 from retry import retry
@@ -17,7 +19,7 @@ class EncounterInsuranceWindow(NextGenWindow):
     """EncounterInsurance class with methods to interact with encounter insurance window."""
 
     @property
-    def window(self):
+    def window(self) -> WindowSpecification:
         """Get Encounter Insurance Window."""
         return self.desktop_app.dialog.child_window(title="frmEncounterPayer", control_type="Window")
 
@@ -151,8 +153,7 @@ class EncounterInsuranceWindow(NextGenWindow):
         """
         self.logger.debug("Clicking close on NextGen Alerts")
         alert = self.desktop_app.dialog.child_window(title=title, control_type="Window")
-        if alert.exists(timeout=2, retry_interval=1):
-            alert.set_focus()
+        if alert.exists(timeout=2, retry_interval=0.001):
             with contextlib.suppress(_ctypes.COMError):
                 alert.child_window(title="Close", control_type="Button").click()
                 alert.wait_not("visible", timeout=5)
@@ -185,7 +186,7 @@ class EncounterInsuranceWindow(NextGenWindow):
         alert = self.desktop_app.dialog.child_window(
             title="NextGen Encounter Insurance Selection Alerts", control_type="Window"
         )
-        if alert.exists(timeout=2, retry_interval=1):
+        if alert.exists(timeout=2, retry_interval=0.001):
             alert.set_focus()
             with contextlib.suppress(_ctypes.COMError):
                 alert.child_window(title="Close", control_type="Button").click()
@@ -200,8 +201,10 @@ class EncounterInsuranceWindow(NextGenWindow):
         self.logger.debug(f"Payers options list: {options}")
         return options
 
+    @retry(exceptions=TimeoutError, tries=3, delay=1)
     def click_cancel_insurance_window(self) -> None:
         """Clicks the Cancel button in the Encounter Insurance window."""
         self.logger.debug("Clicking Cancel in Encounter Insurance window")
         with contextlib.suppress(_ctypes.COMError):
             self.window.child_window(auto_id="cmdCancel", control_type="Button").click_input()
+            self.window.wait_not("visible", timeout=5, retry_interval=0.001)
