@@ -17,8 +17,12 @@ if IS_WINDOWS_OS:
     from pywinauto.application import WindowSpecification
     from pywinauto.controls.uia_controls import ListItemWrapper, EditWrapper
     from pywinauto.findwindows import ElementNotFoundError
+    from pywinauto.base_wrapper import ElementNotEnabled
     from pywinauto.keyboard import send_keys
     from pywinauto.timings import wait_until
+
+from t_desktop.decorators import retry_if_pywin_error
+from t_desktop.decorators import capture_screenshot_if_pywin_error
 from t_ocr import Textract
 from t_ocr.pages import TextractPage
 from PIL import Image, ImageFilter
@@ -38,6 +42,7 @@ from t_nextgen.exceptions import (
     TransactionNumberNotFoundError,
     DeductFieldNotFoundException,
     FieldNotFoundException,
+    StatusNotUpdatedException,
 )
 
 
@@ -159,10 +164,10 @@ class PaymentEntryWindow(NextGenWindow):
     def click_open_button(self) -> None:
         """This method clicks on the open button."""
         self.logger.debug("Clicking on the open button.")
-        open_button = self.desktop_app.dialog.child_window(title="_cmdAction_1", control_type="Button")
+        open_button = self.window.child_window(title="_cmdAction_1", control_type="Button")
         if open_button.is_enabled():
             with contextlib.suppress(_ctypes.COMError):
-                open_button.click()
+                open_button.click_input()
 
     def get_service_rows(self) -> list[ListItemWrapper]:
         """This method gets the service rows from the current payment entry window.
@@ -170,7 +175,7 @@ class PaymentEntryWindow(NextGenWindow):
         Returns:
             rows (list): list of service rows
         """
-        pane = self.desktop_app.dialog.child_window(title="lstPayLedger", control_type="Pane")
+        pane = self.window.child_window(title="lstPayLedger", control_type="Pane")
         rows = pane.wrapper_object().descendants(control_type="DataItem")
         self.logger.debug(f"Number of rows in Payment Entry: {len(rows)}")
         return rows
@@ -196,9 +201,70 @@ class PaymentEntryWindow(NextGenWindow):
         codes_inserted = text.split(",")
         for code in codes_inserted:
             if code.strip() not in next_gen_codes:
+                self.skipping_payer_aarp_pop_up_exist()
                 raise LnItemRsnsNotUpdatedException(f"Error trying to insert {text} in Ln Item Rsns field.")
         return row
 
+    @retry(LnItemRsnsNotUpdatedException, tries=3, delay=1)
+    def add_codes_in_ln_item_rsns(self, text: str, row: ListItemWrapper) -> None:
+        """Add one or more codes to Ln Item Rsns without removing existing ones.
+
+        Args:
+            text (str): Comma-separated codes to add (e.g., "PR2,CO45").
+            row (ListItemWrapper): Row to update.
+        """
+        self.logger.debug(f"Adding codes to Ln Item Rsns: {text}")
+        if self.is_payer_a_patient():
+            self.logger.info("The payer is Patient. Ln Item Rsns field is disabled")
+            return
+
+        existing_codes = self._get_existing_ln_item_rsns_codes(row)
+        new_codes = [c.strip().upper() for c in text.split(",") if c.strip()]
+        combined_codes = existing_codes + [c for c in new_codes if c not in existing_codes]
+
+        if combined_codes == existing_codes:
+            self.logger.debug("Ln Item Rsns already contains requested codes; nothing to do.")
+            return row
+
+        combined_text = ",".join(combined_codes)
+
+        self.remove_selected_items_from_ln_combobox(row)
+        self.set_text_in_specific_row(row, "Ln Item Rsns", combined_text, is_combobox=True)
+
+        row = self._get_row_by_auto_id(row.automation_id())
+        next_gen_codes = (row.descendants(title="Ln Item Rsns", control_type="Edit")[0].get_value() or "").upper()
+
+        for code in combined_codes:
+            if code.strip().upper() not in next_gen_codes:
+                self.skipping_payer_aarp_pop_up_exist()
+                raise LnItemRsnsNotUpdatedException(f"Error trying to add {text} in Ln Item Rsns field.")
+
+        return row
+
+    def _get_existing_ln_item_rsns_codes(self, row: ListItemWrapper) -> list[str]:
+        """Return the list of currently selected Ln Item Rsns codes for the given row.
+
+        The NextGen control renders values like:
+        'CO45  (Charges exceed fee arrangement); PR2  (Coinsurance amount)'.
+        We parse and return only the raw codes: ['CO45', 'PR2'].
+        """
+        row = self.revalidate_service_row(row)
+        value = row.descendants(title="Ln Item Rsns", control_type="Edit")[0].get_value()
+        if value.strip() == "":
+            return []
+
+        parts = re.split(r"[;,]\s*", value)
+        parts = [p.strip() for p in parts if p.strip()]
+
+        codes: list[str] = []
+        for part in parts:
+            code_text = part.split("(")[0].strip()
+            code_token = code_text.split()[0] if code_text else ""
+            if code_token:
+                codes.append(code_token.upper())
+
+        return codes
+
     def remove_selected_items_from_status_combobox(self, row: ListItemWrapper) -> None:
         """Removes the selected items from the Status combobox.
 
@@ -234,9 +300,10 @@ class PaymentEntryWindow(NextGenWindow):
         payer = mcbo.child_window(control_type="Edit", found_index=0)
         return payer.get_value().lower() == "patient"
 
+    @retry(StatusNotUpdatedException, tries=3, delay=1)
     def select_status_in_dropdown(
         self, status: str = "Appeal", row: ListItemWrapper | None = None, status_key: str = ""
-    ) -> None:
+    ) -> bool:
         """This method selects the specified status in the dropdown.
 
         Args:
@@ -244,6 +311,9 @@ class PaymentEntryWindow(NextGenWindow):
             row (ListItemWrapper): row to select. Will use the first row if not provided
             status_key (str): Shortcut key for setting status (default sets to "Appeal" using "A"). Use
             only if first character of status is unique
+
+        Returns:
+            bool: True if the status was selected, False if it was already set
         """
         self.logger.debug(f"Selecting status in dropdown: {status}")
         self.click_open_button()
@@ -252,8 +322,54 @@ class PaymentEntryWindow(NextGenWindow):
         if status_field.get_value() == status:
             self.logger.debug(f"Status was already set. Value: {status}")
             return
-        pane = self.open_status_dropdown_in_service_row(status_field)
+        pane = self.open_status_dropdown_in_service_row(status_field, row)
+        if len(status.split()) > 2:
+            beginning, end = status.split()[0], status.split()[-1]
+            status = self.get_status_in_dropdown_that_begin_with_and_end_with(beginning, end, pane)
         self.select_status_in_service_row(pane, status, status_key)
+        send_keys("+{TAB}")
+        selected_status = status_field.get_value()
+        if selected_status == "":
+            raise StatusNotUpdatedException(f"Failed to select status {status} in dropdown")
+
+    def get_statuses(self, pane: ListItemWrapper) -> list[str]:
+        """This method gets all the statuses in the status dropdown.
+
+        Args:
+            pane (ListItemWrapper): Pane element containing the status dropdown.
+
+        Returns:
+            list[str]: A list of all the statuses in the dropdown.
+        """
+        combo_box = pane.child_window(control_type="ComboBox")
+        combo_box_element = combo_box.wrapper_object()
+        status_list = []
+        for item in combo_box_element.descendants(control_type="ListItem"):
+            status_list.append(item.window_text())
+        return status_list
+
+    def get_status_in_dropdown_that_begin_with_and_end_with(
+        self, start_with: str, ends_with: str, pane: EditWrapper
+    ) -> str:
+        """This method gets the status in the drop down that begins with the value provided.
+
+        Args:
+            start_with (str): value to be searched for in the status column
+            ends_with (str): value to be searched for in the status column
+            pane (EditWrapper): Represents the pane element of the status field
+
+        Returns:
+            str: The selected status
+        """
+        self.logger.debug(f"Selecting status in dropdown that begins with: {start_with} and ends with {ends_with}")
+
+        combo_box = pane.child_window(control_type="ComboBox")
+        combo_box_element = combo_box.wrapper_object()
+        for item in combo_box_element.descendants(control_type="ListItem"):
+            status = item.window_text()
+            if status.lower().startswith(start_with.lower()) and status.lower().endswith(ends_with.lower()):
+                return status
+        return ValueError(f"Could not find status that begins with: {start_with} and ends with {ends_with}")
 
     def select_status_in_service_row(self, pane: WindowSpecification, status: str, status_key: str = "") -> None:
         """Set status in the status column of current service row.
@@ -274,29 +390,26 @@ class PaymentEntryWindow(NextGenWindow):
             )
             with contextlib.suppress(_ctypes.COMError):
                 pane.click_input()
+                send_keys("{TAB}")
+                send_keys("+{TAB}")
 
-    def open_status_dropdown_in_service_row(self, status_field: EditWrapper) -> EditWrapper:
+    @retry(ElementNotFoundError, tries=3, delay=1)
+    def open_status_dropdown_in_service_row(self, status_field: EditWrapper, row: ListItemWrapper) -> EditWrapper:
         """Open the status dropdown for the provided status field.
 
         Args:
             status_field (EditWrapper): the status field whose dropdown is to be opened
+            row (ListItemWrapper): row to revalidate
 
         Returns:
             WindowSpecification: Represents the pane element of the status field
         """
         self.logger.debug("Opening status dropdown in service row")
-        with contextlib.suppress(_ctypes.COMError):
+        for i in range(3):
+            row = self.revalidate_service_row(row)
+            status_field = self.get_status_field(row)
             status_field.click_input()
         pane = self.desktop_app.dialog.child_window(title="_cboInput_0", control_type="Pane")
-        try:
-            with contextlib.suppress(_ctypes.COMError):
-                pane.click_input()
-        except ElementNotFoundError:
-            with contextlib.suppress(_ctypes.COMError):
-                status_field.click_input()
-            pane = self.desktop_app.dialog.child_window(title="_cboInput_0", control_type="Pane")
-            with contextlib.suppress(_ctypes.COMError):
-                pane.click_input()
         return pane
 
     def get_status_field(self, row: ListItemWrapper) -> EditWrapper:
@@ -362,7 +475,8 @@ class PaymentEntryWindow(NextGenWindow):
         """Clicks the recalc button."""
         if click_recalc_button:
             self.logger.debug("Clicking recalc button in Payment Entry Window.")
-            self.desktop_app.click_button_by_element_name(element_name="_cmdAction_6")
+            with contextlib.suppress(_ctypes.COMError):
+                self.window.child_window(title="_cmdAction_6", control_type="Button").click()
         else:
             self.logger.warning("Skipping recalc button click to prevent changes.")
 
@@ -370,7 +484,8 @@ class PaymentEntryWindow(NextGenWindow):
         """Clicks the save button."""
         if click_save_button:
             self.logger.debug("Clicking save button on Payment Entry Window.")
-            self.desktop_app.click_button_by_element_name(element_name="_cmdAction_2")
+            with contextlib.suppress(_ctypes.COMError):
+                self.window.child_window(title="_cmdAction_2", control_type="Button").click()
             self.verify_and_handle_modal("Please specify a status for each line item payments and adjustment.")
         else:
             self.logger.warning("Clicking cancel in payment entry to prevent changes.")
@@ -379,7 +494,8 @@ class PaymentEntryWindow(NextGenWindow):
     def click_cancel(self) -> None:
         """Clicks the cancel button on the payment entry window."""
         self.logger.debug("Clicking cancel button on Payment Entry Window.")
-        self.desktop_app.click_button_by_element_name(element_name="_cmdAction_3")
+        with contextlib.suppress(_ctypes.COMError):
+            self.window.child_window(title="_cmdAction_3", control_type="Button").click()
 
     def verify_and_handle_modal(self, expected_text: str) -> None:
         """Verifies a modal's text and raises an exception if it matches the expected text.
@@ -387,9 +503,11 @@ class PaymentEntryWindow(NextGenWindow):
         Args:
             expected_text (str): The expected text to verify in the modal.
         """
-        success, modal_text = self.desktop_app.close_modal()
-
-        if success and modal_text.upper() == expected_text.upper():
+        success, modal_text = self.desktop_app.close_modal(timeout=2)
+        if not success:
+            return
+        self.logger.debug(f"Modal appeared. Text: {modal_text}")
+        if modal_text.upper() == expected_text.upper():
             message = f"'{expected_text}' found in the modal. Raising exception."
             self.click_cancel()
             raise ServiceLineMissingRequiredFieldError(message)
@@ -404,11 +522,12 @@ class PaymentEntryWindow(NextGenWindow):
             text (str): text to be inserted
         """
         self.logger.debug(f"Setting text in Adj field: {text}")
-        items = row.descendants(control_type="Edit")
+        items = row.children(control_type="Edit")
         for item in items:
             window_text = item.window_text()
             if "adj" in window_text.lower():
                 self.set_text_in_specific_row(row, window_text, text)
+                self.skipping_payer_aarp_pop_up_exist()
                 return
         raise AdjFieldNotFoundException("Adj field not found in the row")
 
@@ -469,9 +588,7 @@ class PaymentEntryWindow(NextGenWindow):
         Returns:
             str: transaction notes text
         """
-        transaction_note_input = self.desktop_app.dialog.child_window(
-            auto_id="txtTransactionNotes", control_type="Edit"
-        )
+        transaction_note_input = self.window.child_window(auto_id="txtTransactionNotes", control_type="Edit")
         notes_value = transaction_note_input.get_value()
         return notes_value
 
@@ -575,6 +692,7 @@ class PaymentEntryWindow(NextGenWindow):
                 return balance.get_value()
         raise BalanceFieldNotFoundException("Balance field not found")
 
+    @retry(ElementNotFoundError, tries=3, delay=0.5)
     def remove_selected_items_from_ln_combobox(self, row: ListItemWrapper) -> None:
         """Removes the selected items from the Ln Item Rsns combobox.
 
@@ -583,21 +701,12 @@ class PaymentEntryWindow(NextGenWindow):
         """
         row = self.revalidate_service_row(row)
         _input = row.descendants(title="Ln Item Rsns", control_type="Edit")[0]
-
         if _input.get_value() == "":
             return
         with contextlib.suppress(_ctypes.COMError):
+            _input.double_click_input()
+            _input.click_input()
             _input.click_input()
-        pane = self.window.child_window(title="_cboInput_0", control_type="Pane")
-        try:
-            with contextlib.suppress(_ctypes.COMError):
-                pane.click_input()
-        except ElementNotFoundError:
-            with contextlib.suppress(_ctypes.COMError):
-                _input.click_input()
-            pane = self.window.child_window(title="_cboInput_0", control_type="Pane")
-            with contextlib.suppress(_ctypes.COMError):
-                pane.click_input()
         with contextlib.suppress(_ctypes.COMError):
             self.desktop_app.app.window(title="MainForm").child_window(
                 title="", control_type="Window"
@@ -607,14 +716,19 @@ class PaymentEntryWindow(NextGenWindow):
 
     def maximize_window(self) -> None:
         """Maximize the Payment Entry Window."""
-        if not self.window.is_maximized():
-            self.logger.debug("Maximizing Payment Entry Window.")
-            self.window.maximize()
+        self.logger.debug("Maximizing Payment Entry Window.")
+        self.window.maximize()
 
+    @retry_if_pywin_error(retries=2, delay=5)
+    @capture_screenshot_if_pywin_error()
     def click_new_button(self) -> None:
         """Click New Button."""
         self.logger.debug("Clicking New Button.")
-        self.desktop_app.click_button_by_element_name(element_name="_cmdAction_0")
+        with contextlib.suppress(_ctypes.COMError):
+            self.window.child_window(title="_cmdAction_0", control_type="Button").click()
+            self.desktop_app.wait_until_element_visible(
+                title="_txtActiveSearch_103", control_type="Edit", timeout=30, retries=3
+            )
 
     def enter_text_in_enc_clm_field(self, text: str) -> None:
         """This method inserts text in the Enc/Clm #.
@@ -624,8 +738,7 @@ class PaymentEntryWindow(NextGenWindow):
         """
         self.logger.debug(f"Inserting text in Enc/Clm #: {text}")
         with contextlib.suppress(_ctypes.COMError):
-            container = self.desktop_app.dialog.child_window(auto_id="vseContainer", control_type="Text")
-            container.set_focus()
+            container = self.window.child_window(auto_id="vseContainer", control_type="Text")
             self.desktop_app.set_text(container.child_window(title="_txtActiveSearch_103", control_type="Edit"), text)
             send_keys("{ENTER}")
 
@@ -637,12 +750,11 @@ class PaymentEntryWindow(NextGenWindow):
         """
         self.logger.debug(f"Inserting text in Tracking #: {text}")
         with contextlib.suppress(_ctypes.COMError):
-            container = self.desktop_app.dialog.child_window(auto_id="vseContainer", control_type="Text")
-            container.set_focus()
+            container = self.window.child_window(auto_id="vseContainer", control_type="Text")
             self.desktop_app.set_text(container.child_window(title="_txtDataEntry_105", control_type="Edit"), text)
             send_keys("{ENTER}")
 
-    def click_close_nextgen_alerts(self, timeout: int = 5, retry_interval: float = 1) -> None:
+    def click_close_nextgen_alerts(self, timeout: int = 5, retry_interval: float = 0.001) -> None:
         """Click close button on NextGen Alerts.
 
         Args:
@@ -652,20 +764,26 @@ class PaymentEntryWindow(NextGenWindow):
         alert = self.desktop_app.dialog.child_window(title="NextGen Alerts", control_type="Window")
         if alert.exists(timeout=timeout, retry_interval=retry_interval):
             self.logger.debug("Clicking close on NextGen Alerts")
-            alert.set_focus()
             with contextlib.suppress(_ctypes.COMError):
                 alert.child_window(title="Close", control_type="Button").click()
 
-    def click_yes_encounter_in_history_status_pop_up(self) -> None:
-        """Click yes button in the encounter in history status pop-up."""
-        if self.desktop_app.dialog.child_window(
+    def click_yes_encounter_in_history_status_pop_up(self, timeout: int = 1, window_title: str = "NextGen") -> None:
+        """Click yes button in the encounter in history status pop-up.
+
+        Args:
+            timeout (int, optional): Time to wait for the pop-up to appear. Defaults to 1.
+            window_title (str, optional): Title of the pop-up window. Defaults to "NextGen".
+        """
+        alert_window = self.desktop_app.dialog.child_window(title=window_title, control_type="Window")
+
+        if alert_window.child_window(
             title="The encounter is in a history status.  Posting a transaction(s) against this encounter "
             "will reinstate the encounter to a billed status.  Do you want to continue?",
             control_type="Text",
-        ).exists(timeout=1, retry_interval=1):
+        ).exists(timeout=timeout, retry_interval=0.001):
             self.logger.debug("Clicking Yes in Encounter in History Status pop-up")
             with contextlib.suppress(_ctypes.COMError):
-                self.desktop_app.dialog.child_window(title="Yes", control_type="Button").click_input()
+                alert_window.child_window(title="Yes", control_type="Button").click()
 
     def _capture_pages_data_as_image_in_window(self) -> Image:
         """Capture Pages Data as Image.
@@ -677,6 +795,28 @@ class PaymentEntryWindow(NextGenWindow):
         custom_control = container.child_window(auto_id="_lblDisplayField_0", control_type="Text")
         return custom_control.capture_as_image()
 
+    def get_pay_column_header_text(self) -> str:
+        """Get the text of the Pay column header.
+
+        Returns:
+            str: The Pay column header text.
+        """
+        pay_column_header = self.window.child_window(auto_id="[Column Header] Column18", control_type="HeaderItem")
+        pay_column_header_text = pay_column_header.window_text()
+        self.logger.debug(f"Pay column header text: {pay_column_header_text}")
+        return pay_column_header_text
+
+    def get_adj_column_header_text(self) -> str:
+        """Get the text of the Adj column header.
+
+        Returns:
+            str: The Adj column header text.
+        """
+        adj_column_header = self.window.child_window(auto_id="[Column Header] Column19", control_type="HeaderItem")
+        adj_column_header_text = adj_column_header.window_text()
+        self.logger.debug(f"Adj column header text: {adj_column_header_text}")
+        return adj_column_header_text
+
     def get_pages_data(self, screenshot_destination: Path) -> str:
         """Extract Pages Dataw.
 
@@ -729,14 +869,14 @@ class PaymentEntryWindow(NextGenWindow):
         raise TransactionNumberNotFoundError("Transaction number not found in the page data.")
 
     def click_shield_icon(self) -> None:
-        """Click shield icon in pyament entry window."""
+        """Click shield icon in payment entry window."""
         self.logger.debug("Clicking shield icon in Payment Entry Window.")
         pnl_small_tool = self.window.child_window(title="pnlSmallTool", control_type="Text")
         with contextlib.suppress(_ctypes.COMError):
             pnl_small_tool.child_window(auto_id="_cmdSmallTool_03", control_type="Button").click_input()
 
     def click_balance_control_icon(self) -> None:
-        """Click balance control icon in pyament entry window."""
+        """Click balance control icon in payment entry window."""
         self.logger.debug("Clicking balance control icon in Payment Entry Window.")
         pnl_small_tool = self.window.child_window(title="pnlSmallTool", control_type="Text")
         with contextlib.suppress(_ctypes.COMError):
@@ -749,15 +889,17 @@ class PaymentEntryWindow(NextGenWindow):
             payer (str): payer to be selected
         """
         self.logger.debug(f"Selecting payer in dropdown: {payer}")
-        container = self.window.child_window(auto_id="vseContainer", control_type="Text")
-        pane = container.child_window(title="_cboInput_104", control_type="Pane")
-        combo_box = pane.child_window(auto_id="mcbo", control_type="ComboBox")
+        container = self.window.child_window(auto_id="vseContainer", control_type="Text", top_level_only=True)
+        pane = container.child_window(title="_cboInput_104", control_type="Pane", top_level_only=True)
+        combo_box = pane.child_window(auto_id="mcbo", control_type="ComboBox", top_level_only=True)
         pane_object = pane.wrapper_object()
         options = pane_object.descendants(control_type="ListItem")
         for option in options:
             value = option.window_text()
             if payer.lower() in value.lower():
-                self.desktop_app.set_text(combo_box.child_window(control_type="Edit", found_index=0), value)
+                self.desktop_app.set_text(
+                    combo_box.child_window(control_type="Edit", top_level_only=True, found_index=0), value
+                )
                 return
 
     def get_payers_from_dropdown(self) -> list:
@@ -774,7 +916,7 @@ class PaymentEntryWindow(NextGenWindow):
     def set_text_in_allowed(self, row: ListItemWrapper, allowed_amt: Decimal) -> None:
         """Update the Allowed field for a specific service line."""
         self.logger.debug(f"Setting Allowed value in specific row: {allowed_amt}")
-        items = row.descendants(control_type="Edit")
+        items = row.children(control_type="Edit")
         allowed_element_name = None
         for item in items:
             window_text = item.window_text()
@@ -794,7 +936,7 @@ class PaymentEntryWindow(NextGenWindow):
         self.logger.debug(f"Setting text in Pay field: {text}")
         with contextlib.suppress(_ctypes.COMError):
             str_text = str(convert_to_decimal(text))
-            items = row.descendants(control_type="Edit")
+            items = row.children(control_type="Edit")
             payment_element_name = None
             for item in items:
                 window_text = item.window_text()
@@ -805,15 +947,14 @@ class PaymentEntryWindow(NextGenWindow):
                     return
             raise PayFieldNotFoundException("Pay field not found in the row")
 
-    def skipping_payer_aarp_pop_up_exist(self) -> bool:
+    def skipping_payer_aarp_pop_up_exist(self, timeout: int = 1) -> bool:
         """This method checks if the skipping payer AARP pop up exists. If it does, it clicks on the ok button.
 
         Returns:
             bool: True if the pop up exists, False otherwise
         """
-        if self.desktop_app.dialog.child_window(auto_id="65535", control_type="Text").exists(
-            timeout=3, retry_interval=1
-        ):
+        alert_window = self.desktop_app.dialog.child_window(auto_id="65535", control_type="Text")
+        if alert_window.exists(timeout=timeout, retry_interval=0.001):
             with contextlib.suppress(_ctypes.COMError):
                 self.desktop_app.dialog.child_window(title="OK", control_type="Button").click()
             self.logger.debug("Clicked on OK button in Payer AARP pop up.")
@@ -880,7 +1021,7 @@ class PaymentEntryWindow(NextGenWindow):
             self.enter_text_in_tracking_field(payment_trn)
             self.logger.debug("Payment window cleaned and tracking number entered successfully.")
 
-    def select_pay_code_in_drop_down(self, option_to_select: str):
+    def select_pay_code_in_drop_down(self, option_to_select: str) -> None:
         """This method selects the pay code in the drop down.
 
         Args:
@@ -898,7 +1039,7 @@ class PaymentEntryWindow(NextGenWindow):
                 self.desktop_app.set_text(combo_box.child_window(control_type="Edit", found_index=0), value)
                 return
 
-    def select_adj_code_in_drop_down(self, option_to_select: str):
+    def select_adj_code_in_drop_down(self, option_to_select: str) -> None:
         """Select the adjustment code from the dropdown.
 
         Args:
@@ -915,7 +1056,7 @@ class PaymentEntryWindow(NextGenWindow):
                 self.desktop_app.set_text(combo_box.child_window(control_type="Edit", found_index=0), value)
                 return
 
-    def close_payment_entry_window(self):
+    def close_payment_entry_window(self) -> None:
         """Click close payment entry window."""
         title_bar = self.window.child_window(control_type="TitleBar")
         title_bar.child_window(title="Close", control_type="Button").click_input()
@@ -927,48 +1068,50 @@ class PaymentEntryWindow(NextGenWindow):
             control_number (str): _description_
         """
         self.desktop_app.set_text(
-            self.desktop_app.dialog.child_window(title="_txtDataEntry_116", control_type="Edit"), control_number
+            self.window.child_window(title="_txtDataEntry_116", control_type="Edit", top_level_only=True),
+            control_number,
         )
 
     def click_show_all_lines_check_box(self) -> None:
         """This method clicks on the show all lines check box."""
-        check_box = self.desktop_app.dialog.child_window(title="chkShowAll", control_type="CheckBox")
+        check_box = self.window.child_window(title="chkShowAll", control_type="CheckBox")
         self.desktop_app.toggle_checkbox(check_box)
 
-    def check_bad_debt(self) -> bool:
+    def check_bad_debt(self, timeout: int = 2, window_title: str = "NextGen") -> bool:
         """This method checks if the encounter is pre-listed for bad debt.
 
         Returns:
             bool: True if the pop up exists, False otherwise
         """
-        bad_debt_popup = self.desktop_app.dialog.child_window(title="NextGen", control_type="Window")
-        bad_debt_text = self.desktop_app.dialog.child_window(
-            title="This Encounter is pre-listed for bad debt or in bad debt.  You may not edit any information."
+        alert_window = self.desktop_app.dialog.child_window(title=window_title, control_type="Window")
+
+        bad_debt_text = alert_window.child_window(
+            title="This encounter is pre-listed for bad debt or in bad debt.  You may not edit any information.",
+            control_type="Text",
         )
-        if bad_debt_text.exists():
-            bad_debt_popup.child_window(title="OK", control_type="Button").click_input()
-            balance_window = self.desktop_app.dialog.child_window(title="BalanceControl", control_type="Window").wait(
-                "visible", 10
-            )
-            balance_window.child_window(title="cmdCancel", control_type="Button").click_input()
+        if bad_debt_text.exists(timeout=timeout, retry_interval=0.001):
+            alert_window.child_window(title="OK", control_type="Button").click_input()
             return True
         return False
 
-    def check_invalid_encounter_pop_up_exist(self) -> bool:
+    def check_invalid_encounter_pop_up_exist(self, timeout: int = 2, window_title: str = "NextGen") -> bool:
         """This method checks if the invalid encounter pop up exists.
 
+        Args:
+            timeout (int, optional): Time to wait for the pop-up to appear. Defaults to 2.
+
         Returns:
             bool: True if the pop up exists, False otherwise
         """
-        invalid_encounter_popup = self.desktop_app.dialog.child_window(title="NextGen", control_type="Window")
-        invalid_encounter_text_1 = self.desktop_app.dialog.child_window(
-            title="The Encounter number entered is invalid."
+        alert_window = self.desktop_app.dialog.child_window(title=window_title, control_type="Window")
+
+        invalid_encounter_text_1 = alert_window.child_window(
+            title="The Encounter number entered is invalid.", control_type="Text", top_level_only=True
         )
-        invalid_encounter_text_2 = self.desktop_app.dialog.child_window(title="Invalid encounter number!")
-        if invalid_encounter_text_1.exists(timeout=3, retry_interval=1) or invalid_encounter_text_2.exists(
-            timeout=1, retry_interval=1
-        ):
-            invalid_encounter_popup.child_window(title="OK", control_type="Button").click_input()
+        if invalid_encounter_text_1.exists(timeout=timeout, retry_interval=0.001) or alert_window.child_window(
+            title="Invalid encounter number!", control_type="Text"
+        ).exists(timeout=1, retry_interval=0.001):
+            alert_window.child_window(title="OK", control_type="Button").click()
             return True
         return False
 
@@ -994,17 +1137,98 @@ class PaymentEntryWindow(NextGenWindow):
         Returns:
             str: value
         """
-        pane = self.desktop_app.dialog.child_window(auto_id="_cboInput_103", control_type="Pane")
-        combo_box = pane.child_window(control_type="ComboBox")
+        pane = self.desktop_app.dialog.child_window(auto_id="_cboInput_103", control_type="Pane", top_level_only=True)
+        combo_box = pane.child_window(auto_id="mcbo", control_type="ComboBox")
         edit_control = combo_box.child_window(control_type="Edit", found_index=0)
         enc_clm = edit_control.get_value()
         return enc_clm
 
     def get_resub_value(self) -> str:
-        """This method gets the value from the resub field.
+        """This method quickly gets the value from the resub field.
 
         Returns:
-            str: resub value
+            str | None: resub value if found, otherwise None
         """
-        resub = self.desktop_app.dialog.child_window(title="_txtDataEntry_116", control_type="Edit").get_value()
-        return resub
+        edit = self.desktop_app.dialog.child_window(title="PaymentEntry", control_type="Window").child_window(
+            auto_id="_txtDataEntry_116", control_type="Edit"
+        )
+        return str(edit.get_value())
+
+    def payment_entry_window_opened_in_encounter(self, num: str, resub: str) -> bool:
+        """Check if Payment Entry is opened for a given encounter number and resub value.
+
+        Args:
+            num (str): Encounter number to verify.
+            resub (str): Expected resub value.
+
+        Returns:
+            bool: True if Payment Entry matches both encounter and resub, else False.
+        """
+        self.logger.debug(f"Checking if payment entry window is opened for encounter: {num}")
+        try:
+            enc_clm = self.get_enc_clm_edit_value()
+        except (ElementNotFoundError, ElementNotEnabled, TimeoutError):
+            self.logger.info("Payment Entry window not opened or encounter field not readable.")
+            return False
+
+        if not enc_clm or num not in enc_clm:
+            return False
+
+        resub_value = self.get_resub_value()
+        if str(resub) != str(resub_value):
+            self.logger.debug(f"Resub mismatch or not readable: expected={resub!r}, got={resub_value!r}")
+            return False
+
+        self.logger.debug("Payment Entry is opened for the expected encounter and resub.")
+        return True
+
+    def delete_allowed_field(self, row: WindowSpecification) -> None:
+        """Delete (set to empty) the Allowed field of the given service line."""
+        self.logger.debug("Deleting Allowed field (set to empty).")
+        row = self.revalidate_service_row(row)
+        items = row.children(control_type="Edit")
+        for item in items:
+            window_text = item.window_text()
+            if "allowed" in window_text.lower():
+                element = row.descendants(title=window_text, control_type="Edit")[0]
+                element.click_input()
+                send_keys("^a{DELETE}")
+                if element.get_value() != "":
+                    element.click_input()
+                    send_keys("^a{DELETE}")
+                return
+        raise FieldNotFoundException("Allowed field not found in the row")
+
+    def delete_adj_field(self, row: WindowSpecification) -> None:
+        """Delete (set to empty) the Adj field of the given service line."""
+        self.logger.debug("Deleting Adj field (set to empty).")
+        row = self.revalidate_service_row(row)
+        items = row.children(control_type="Edit")
+        for item in items:
+            window_text = item.window_text()
+            if "adj" in window_text.lower():
+                element = row.descendants(title=window_text, control_type="Edit")[0]
+                element.click_input()
+                send_keys("^a{DELETE}")
+                if element.get_value() != "":
+                    element.click_input()
+                    send_keys("^a{DELETE}")
+                return
+        raise AdjFieldNotFoundException("Adj field not found in the row")
+
+    def delete_pay_field(self, row: WindowSpecification) -> None:
+        """Delete (set to empty) the Pay field of the given service line."""
+        self.logger.debug("Deleting Pay field (set to empty).")
+        row = self.revalidate_service_row(row)
+        items = row.children(control_type="Edit")
+        for item in items:
+            window_text = item.window_text()
+            if "pay" in window_text.lower():
+                element = row.descendants(title=window_text, control_type="Edit")[0]
+                element.click_input()
+                send_keys("^a{DELETE}")
+                if element.get_value() != "":
+                    element.click_input()
+                    send_keys("^a{DELETE}")
+                return
+        raise PayFieldNotFoundException("Pay field not found in the row")
